// SubRandom.inc
// version 1.0
//
// an include containing macros to produce sub-random number sequences
//
// by Michael Andrews, Feb 2003

#ifndef(_SUB_RANDOM_INC)
#declare _SUB_RANDOM_INC = version;
#version 3.5;

#ifdef(View_POV_Include_Stack)
   #debug "including subrandom.inc\n"
#end

#macro Halton(A,K,L,Off)

  #local Base = 2.0;
  #local I=A+15; // pow(Base,4)-1;
  #declare K = 0;
  #local Half = 1.0 / Base;

  #while ( I>0 )
    #local Digit = mod ( I, Base );
    #declare K = K + Digit * Half;
    #local I = ( I - Digit ) / Base;
    #local Half = Half / Base;
  #end

  #local Base = 3.0;
  #local I=A+80+Off; // pow(Base,4)-1;
  #declare L = 0;
  #local Half = 1.0 / Base;

  #while ( I>0 )
    #local Digit = mod ( I, Base );
    #declare L = L + Digit * Half;
    #local I = ( I - Digit ) / Base;
    #local Half = Half / Base;
  #end

#end

#macro Hammersley(A,N,K,L,Off)

  #declare K = (A+0.5)/N;

  #local Base = 2.0;
  #local I=A+15+Off; // pow(Base,4)-1;
  #declare L = 0;
  #local Half = 1.0 / Base;

  #while ( I>0 )
    #local Digit = mod ( I, Base );
    #declare L = L + Digit * Half;
    #local I = ( I - Digit ) / Base;
    #local Half = Half / Base;
  #end

#end

#macro Golden(A,N,K,L,Off)
  #declare G_PHI = (sqrt(5)-1)/2;

  #declare K = (A+0.5)/N;

  #local I=A+Off;
  #declare L = mod(I*G_PHI,1);

#end

// fill array Pos[] with specified random sequence in particular distribution
#macro Sub_Random_Dist(Pos,Type,Dist,Off)

  #local maxC = dimension_size(Pos,1);

  #local C = 0; #while (C < maxC)
    #declare Pos[C] = x*0;
    #local D1 = 0;
    #local D2 = 0;
    #switch (Type)
      #case (0) Halton(C,D1,D2,Off) #break
      #case (1) Hammersley(C,maxC,D1,D2,Off) #break
      #case (2) Golden(C,maxC,D1,D2,Off) #break
      #else #error "Unknown sequence type requested in Sub_Random_Dist()."
    #end
    #switch (Dist)
      #case (0) // Square [0,1) by [0,1)
        #declare Pos[C] = <D1,D2,0>;
      #break
      #case (1) // Unit disk
        #declare Pos[C] = sqrt(D1)*<cos(2*pi*D2),-sin(2*pi*D2),0>;
      #break
      #case (2) // Unit spherical shell
        #local PH = 180*(acos(1-2*D1)/pi);
        #local TH = 360*D2;
        #declare Pos[C] = vrotate(y, <PH, TH, 0>);
      #break
      #else
        #error "Unknown distribution requested in Sub_Random_Dist()."
    #end
  #local C = C + 1; #end

#end

#version _SUB_RANDOM_INC;
#end // end _SUB_RANDOM_INC
